//StringTable.h

#ifndef _STRING_TABLE_H
#define _STRING_TABLE_H

#include <stdio.h>
#include <vector>
#include <string>



struct StringTable
{
	typedef std::vector<std::string>	Row;
	typedef std::vector<Row>			Table;
	Table	table;

	~StringTable()
	{
		table.clear();
	}

	//row y, col x
	void Assign( int nRow, int nCol )
	{
		Row	row( nCol );
		table.reserve( nRow );
		table.assign( nRow, row );
	}

	void IncRow()
	{
		Row row( (int)table[0].size() ); 
		table.push_back( row );
	}

	void IncColumn()
	{
		int nSize = (int)table.size();
	
		for( int n = 0; n < nSize; ++n )
		{
			Row& row = table[n];
			row.push_back( std::string() );
		}
	}

	int GetRowCount()
	{
		int m = (int)table.size();
		return m;
	}

	int GetColumnCount()
	{
		int rowSize = (int)table.size();
		
		int nMax = 0;
		for( int n = 0; n < rowSize; ++n )
		{
			int nSize = (int)table[n].size();
			if( nSize > nMax )
				nMax = nSize;
		}
		return nMax;
	}

	int GetColumnCount( int nRow )
	{
		int n = (int)table[nRow].size();
		return n;
	}

	std::string* Get( int nRow, int nCol )
	{
		int m = (int)table.size();
		if( nRow > m )
			return NULL;
		
		int n = (int)table[nRow].size();
		if( nCol > n )
			return NULL;

		Row& row = table[nRow];
		return ( &row[nCol] );
	}

	bool Set( int nRow, int nCol, const char* str )
	{
		int m = (int)table.size();
		int n = (int)table[0].size();

		if( nRow > m || nCol > n )
			return false;
		Row& row = table[nRow];
		row[nCol] = str;
		return true;
	}
};


struct StringBoolTable
{
	struct	StringSt
	{
		std::string string;
		bool bDirty;

		StringSt()
		{
			bDirty = false;
		}

		~StringSt()
		{
			string.clear();
		}

		void Set( const char* str )
		{
			string = str;
			bDirty = true;
		}
	};

	typedef std::vector<StringSt>		Row;
	typedef std::vector<Row>			Table;
	Table	table;

	~StringBoolTable()
	{
		Clear();
	}

	//row y, col x
	void Assign( int nRow, int nCol )
	{
		Row	row( nCol );
		table.reserve( nRow );
		table.assign( nRow, row );
	}

	void IncRow()
	{
		Row row( (int)table[0].size() ); 
		table.push_back( row );
	}

	void IncColumn()
	{
		int nSize = (int)table.size();
	
		for( int n = 0; n < nSize; ++n )
		{
			Row& row = table[n];
			row.push_back( StringSt() );
		}
	}

	int GetRowCount()
	{
		int m = (int)table.size();
		return m;
	}

	int GetColumnCount()
	{
		int rowSize = (int)table.size();
		
		int nMax = 0;
		for( int n = 0; n < rowSize; ++n )
		{
			int nSize = (int)table[n].size();
			if( nSize > nMax )
				nMax = nSize;
		}
		return nMax;
	}

	int GetColumnCount( int nRow )
	{
		int n = (int)table[nRow].size();
		return n;
	}

	StringSt* Get( int nRow, int nCol )
	{
		int m = (int)table.size();
		if( nRow > m )
			return NULL;
		
		int n = (int)table[nRow].size();
		if( nCol > n )
			return NULL;

		Row& row = table[nRow];
		return ( &row[nCol] );
	}

	bool Set( int nRow, int nCol, const char* str, bool bDirty )
	{
		int m = (int)table.size();
		int n = (int)table[0].size();
		if( nRow == -1 )
			nRow = m - 1;

		if( nRow > m || nCol > n )
			return false;
		Row& row = table[nRow];
		row[nCol].string = str;
		row[nCol].bDirty = bDirty;
		return true;
	}

	void Clear()
	{
		table.clear();
	}
};


#endif